#include <TimerOne.h>

#include <Adafruit_NeoPixel.h>
#include <Servo.h>

/**
 * Activity Board code for custom made board
 * 
 * http://www.idreamincode.com/2015/12/19/activity-board-for-my-niece/
 */
#ifdef __AVR__
  #include <avr/power.h>
#endif

#define buttonRed     2
#define buttonGreen   3
#define buttonBlue    4
#define servoPIN      5
#define lightsPIN     6
#define breakBeam     7
#define paddleSwitch  8
#define counterFan    9
#define footPedal     10
#define fanPIN        12
#define ethTopGreen   A0
#define ethBotRed     A1

#define NearFarPIN    A5


int sensorState = 0, lastState=0; 
int brokenBeam  = 0;
int NearFarVal  = 0;
Servo myservo; 
int pos = 0;
const int numReadings = 300;
int nearFarRunningAverage[numReadings];
int NFreadIndex = 0;              // the index of the current reading
long NFtotal = 0;                  // the running total
int NFaverage = 0;                // the average

long msLastUpdate = 0;
long msLastUpdateLights = 0;

// RGB colors
float RGBRed[4];
float RGBGreen[4];
float RGBBlue[4];
float RGBRateIncrease = 0.1;
float RGBRateDecrease[4] ={ 1.5, 0.8, 0.0 ,0.8};  // every 10 milliseconds
float RGBConstrain[4]    = {50,255,255,255};

int footPedalVal    = 0;
int footPedalLast   = 0;
int paddleSwitchVal = 0;
int paddleSwitchLast= 0;

// Parameter 1 = number of pixels in strip
// Parameter 2 = Arduino pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(4, lightsPIN, NEO_GRB + NEO_KHZ800);

// IMPORTANT: To reduce NeoPixel burnout risk, add 1000 uF capacitor across
// pixel power leads, add 300 - 500 Ohm resistor on first pixel's data input
// and minimize distance between Arduino and first pixel.  Avoid connecting
// on a live circuit...if you must, connect GND first.

void setup() {

  pinMode(buttonRed,    INPUT);
  pinMode(buttonGreen,  INPUT);
  pinMode(buttonBlue,   INPUT);
  
  pinMode(breakBeam,    INPUT);
  digitalWrite(breakBeam, HIGH);

  pinMode(paddleSwitch, INPUT_PULLUP);
  pinMode(footPedal,    INPUT);
  pinMode(NearFarPIN,   INPUT);

  pinMode(ethTopGreen,  INPUT_PULLUP);
  pinMode(ethBotRed,    INPUT_PULLUP); 
  
  pinMode(counterFan, OUTPUT);
  digitalWrite(counterFan, LOW);
  pinMode(fanPIN, OUTPUT);
  digitalWrite(fanPIN, LOW); 

  // I think this Timer1 was removed
  Timer1.initialize(1000000);
  Timer1.attachInterrupt(increaseCounter); // Increase counter every 100 milliseconds
  
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'

  myservo.attach(servoPIN, 500, 2200);
  Serial.begin(9600);

  for (int i = 0; i < numReadings; i++) {
    nearFarRunningAverage[i] = 0;
  }
  //colorWipe(strip.Color(255, 0, 0), 50); // Red
}

void loop() {
 // digitalWrite(fanPIN, HIGH);  

  
  // read the state of the pushbutton value:
  sensorState = digitalRead(breakBeam);
 
  // --------------------
  // Running Average for NearFar
  NFtotal = NFtotal - nearFarRunningAverage[NFreadIndex];
  // read from the sensor:
  nearFarRunningAverage[NFreadIndex] = analogRead(NearFarPIN);
  if(nearFarRunningAverage[NFreadIndex] < 80){
     nearFarRunningAverage[NFreadIndex] = 0;  
  }
  //Serial.println(nearFarRunningAverage[NFreadIndex]);
  // add the reading to the total:
  NFtotal = NFtotal + nearFarRunningAverage[NFreadIndex];
  // advance to the next position in the array:
  NFreadIndex = NFreadIndex + 1;

  // if we're at the end of the array...
  if (NFreadIndex >= numReadings) {
    // ...wrap around to the beginning:
    NFreadIndex = 0;
  }

  // calculate the average:
  NFaverage = NFtotal / numReadings;  
  // end Running Average for NearFar
  // --------------------

  
  myservo.write(map(NFaverage,0,256,160,0));
   
  // check if the sensor beam is broken
  // if it is, the sensorState is LOW:
  if (sensorState == LOW) {     
    // turn LED on:
    //digitalWrite(LEDPIN, HIGH);  
    digitalWrite(fanPIN, HIGH);  
  } 
  else {
    digitalWrite(fanPIN, LOW);  
    // turn LED off:
    //digitalWrite(LEDPIN, LOW); 
  }
  
  if (sensorState && !lastState) {
    Serial.println("Unbroken");
    brokenBeam  = 0;
  } 
  if (!sensorState && lastState) {
    Serial.println("Broken");
    brokenBeam  = 1;
  }
  lastState = sensorState;

  if(digitalRead(buttonRed)){
    RGBRed[0] = RGBRed[0] + RGBRateIncrease;

  }
  if(digitalRead(buttonGreen)){
    RGBGreen[0] = RGBGreen[0] + RGBRateIncrease;
   
  }
  if(digitalRead(buttonBlue)){
    RGBBlue[0] = RGBBlue[0] + RGBRateIncrease;
  }
  footPedalVal = digitalRead(footPedal);
  if(footPedalVal == 1 && footPedalLast == 0){
    RGBRed[1] = random(0,RGBConstrain[1]) ; 
    RGBGreen[1] = random(0,RGBConstrain[1]) ;
    RGBBlue[1] = random(0,RGBConstrain[1]) ;
    
  }
  footPedalLast = footPedalVal;

  // Paddle Switch code
  paddleSwitchVal = digitalRead(paddleSwitch);
  if(paddleSwitchVal == 0 && paddleSwitchLast == 1){
    RGBRed[2]   = random(2) * 255; 
    RGBGreen[2] = random(2) * 255;
    RGBBlue[2]  = random(2) * 255 ;
    if(RGBRed[2] + RGBGreen[2] + RGBBlue[2] == 0 ){
      RGBRed[2]   =  255; 
      RGBGreen[2] =  255;
      RGBBlue[2]  =  255 ;      
    }
  }
  if(paddleSwitchVal == 1){
    RGBRed[2]   = 0;
    RGBGreen[2] = 0;
    RGBBlue[2]  = 0;
  }
  paddleSwitchLast = paddleSwitchVal;
  // end paddleswitch

  // Ethernet plug
  if(digitalRead(ethTopGreen) == 0){
    RGBRed[3]   = 0;
    RGBGreen[3] = 255;
    RGBBlue[3]  = 0;   
  }
  if(digitalRead(ethBotRed) == 0){
    RGBRed[3]   = 255;
    RGBGreen[3] = 0;
    RGBBlue[3]  = 0;      
  }
  if( digitalRead(ethTopGreen) == 1 && digitalRead(ethBotRed) == 1 ){
    RGBRed[3]   = 0;
    RGBGreen[3] = 0;
    RGBBlue[3]  = 0; 
  }
  
  if(millis() - msLastUpdate > 100){
    msLastUpdate = millis();
    increaseCounter();
  }
  if(millis() - msLastUpdateLights > 10){
    for(int i=0;i<strip.numPixels(); i++){
      RGBRed[i]    = constrain( RGBRed[i]  ,0,RGBConstrain[i]);
      RGBGreen[i]  = constrain( RGBGreen[i],0,RGBConstrain[i]);
      RGBBlue[i]   = constrain( RGBBlue[i] ,0,RGBConstrain[i]);

      strip.setPixelColor(i,strip.Color( RGBBlue[i],RGBRed[i],RGBGreen[i] ));
      
      
      RGBRed[i]    = RGBRed[i]    - RGBRateDecrease[i];
      RGBGreen[i]  = RGBGreen[i]  - RGBRateDecrease[i];
      RGBBlue[i]   = RGBBlue[i]   - RGBRateDecrease[i];
    }
    strip.show(); 
    /*Serial.print("Red: \t");
    Serial.print(RGBRed);
    Serial.print("\tGreen: \t");
    Serial.print(RGBGreen);
    Serial.print("\tBlue: \t");
    Serial.println(RGBBlue);
    */
 

    msLastUpdateLights = millis();
  }
  
  
  /*
  for (pos = 0; pos <= 180; pos += 1) { // goes from 0 degrees to 180 degrees
    // in steps of 1 degree
    myservo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(15);                       // waits 15ms for the servo to reach the position
  }
  for (pos = 180; pos >= 0; pos -= 1) { // goes from 180 degrees to 0 degrees
    myservo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(15);                       // waits 15ms for the servo to reach the position
  }
  */
 

}

void increaseCounter(){
  if(brokenBeam == 1){
    digitalWrite(counterFan, HIGH);
    delay(50);
    digitalWrite(counterFan, LOW);
    Serial.println("Counter Increasing");
  }  
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();
    delay(wait);
  }
}

long runningAverage(int M) {
  #define LM_SIZE 10
  static int LM[LM_SIZE];      // LastMeasurements
  static byte index = 0;
  static long sum = 0;
  static byte count = 0;

  // keep sum updated to improve speed.
  sum -= LM[index];
  LM[index] = M;
  sum += LM[index];
  index = index % LM_SIZE;
  if (count < LM_SIZE) count++;

  return sum / count;
}
